/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.client.components;

import static com.google.gwt.query.client.GQuery.$;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.dussan.vaadin.dtable.client.components.base.TableComponent;
import org.dussan.vaadin.dtable.client.components.events.SearchingColumnsEvent;
import org.dussan.vaadin.dtable.client.ui.VDTable;

import com.google.gwt.query.client.Function;
import com.google.gwt.query.client.GQuery;
import com.google.gwt.query.client.css.CSS;
import com.google.gwt.query.client.css.Length;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.Event;
import com.vaadin.terminal.gwt.client.UIDL;

public class SearchingColumns extends TableComponent {

	private String frozenHeaderId = "table#dt-frozen-header-table";
	private String headerId = "table#dt-header-table";

	private Boolean isVisible = true;
	private Boolean show = false;
	private Map<Integer, String> columnsSearching = new HashMap<Integer, String>();

	private Boolean isSearchingMessageSend = false;
	private int markedColumn = -1;
	private String markedValue = new String();
	private String initValue = new String();
	private String focusValue = new String();
	private String eventValue = new String();

	private GQuery frozenInputMouseOverEventHandler = null;
	private GQuery frozenHeaderMouseOverEventHandler = null;
	private GQuery focusFrozenHeaderEventHandler = null;
	private GQuery blurFrozenHeaderEventHandler = null;
	private GQuery frozenHeaderKeydownEventHandler = null;
	private GQuery frozenHeaderKeyupEventHandler = null;
	private GQuery inputMouseOverEventHandler = null;
	private GQuery headerMouseOverEventHandler = null;
	private GQuery focusHeaderEventHandler = null;
	private GQuery blurHeaderEventHandler = null;
	private GQuery headerKeyDownEventHandler = null;
	private GQuery headerKeyUpEventHandler = null;

	public SearchingColumns(VDTable dTable) {
		super(dTable);
	}

	private SearchingColumns getThis() {
		return this;
	}

	private void empty() {
		$(headerId).children("tr").eq(1).remove();
		if (getDTable().config().isEnabledFrozenTable()) {
			$(frozenHeaderId).children("tr").eq(1).remove();
		}
	}

	private void prepare() {
		empty();
		$(headerId).children("tr").eq(0).clone()
				.id(getDTable().getId() + ":1:t").appendTo($(headerId));
		if (getDTable().config().isEnabledFrozenTable()) {
			$(frozenHeaderId).children("tr").eq(0).clone()
					.id(getDTable().getId() + ":1:ft")
					.appendTo($(frozenHeaderId));
		}
	}

	private void appendSearchRow(GQuery table) {
		GQuery trSearch = table.children("tr").eq(1);
		final String idSearch = trSearch.id().substring(0,
				trSearch.id().lastIndexOf(":") + 1);
		table.children("tr").eq(1).children("td").each(new Function() {
			public void f(Element td) {
				int column = Integer.parseInt(td.getId().split(":")[2]);
				String id = idSearch + column;
				$(td).id(id)
						.removeClass("asc", "desc")
						.addClass("search")
						.css(CSS.TEXT_ALIGN, "")
						.children("div")
						.id(id + ":1")
						.removeClass("nowrap", "normal")
						.css(CSS.HEIGHT, "")
						.css(CSS.MARGIN_LEFT.with(Length.px(getDTable()
								.config().getHeaderCellMarginLeft())))
						.css(CSS.MARGIN_RIGHT.with(Length.px(getDTable()
								.config().getHeaderCellMarginRight())))
						.children("span")
						.replaceWith($("<input/>").id(id + ":2"));

				if (getDTable().isFrozen(column)
						&& !getDTable().config().isEnabledFilteredFrozenTable()) {
					$(td).children("div").children("input")
							.addClass("disabled").attr("readonly", "readonly");
				}
			}
		});
	}

	private void appendSearchRows() {
		appendSearchRow($(headerId));
		if (getDTable().config().isEnabledFrozenTable()) {
			appendSearchRow($(frozenHeaderId));
		}
	}

	private void activateFrozenInputMouseOverEventHandler(boolean enable) {
		if (enable) {
			activateFrozenInputMouseOverEventHandler(false);
			frozenInputMouseOverEventHandler = $(frozenHeaderId).children("tr")
					.eq(1).children("td").children("div")
					.delegate("input", "mouseover", new Function() {
						public void f(Element e) {
							SearchingColumnsEvent.mouseOver(getThis(),
									frozenHeaderId, $(e).id());
						}
					}).delegate("input", "mouseout", new Function() {
						public void f(Element e) {
							SearchingColumnsEvent.mouseOut(getThis(),
									frozenHeaderId, $(e).id());
						}
					});
		} else if (!enable && frozenInputMouseOverEventHandler != null) {
			frozenInputMouseOverEventHandler.undelegate("input", "mouseover")
					.undelegate("input", "mouseout");
			frozenInputMouseOverEventHandler = null;
		}
	}

	private void activateFrozenHeaderMouseOverEventHandler(boolean enable) {
		if (enable) {
			activateFrozenHeaderMouseOverEventHandler(false);
			frozenHeaderMouseOverEventHandler = $(frozenHeaderId)
					.children("tr").eq(1)
					.delegate("td", "mouseover", new Function() {
						public void f(Element e) {
							SearchingColumnsEvent.mouseOver(getThis(),
									frozenHeaderId, $(e).id());
						}
					}).delegate("td", "mouseout", new Function() {
						public void f(Element e) {
							SearchingColumnsEvent.mouseOut(getThis(),
									frozenHeaderId, $(e).id());
						}
					});
		} else if (!enable && frozenHeaderMouseOverEventHandler != null) {
			frozenHeaderMouseOverEventHandler.undelegate("td", "mouseover")
					.undelegate("td", "mouseout");
			frozenHeaderMouseOverEventHandler = null;
		}
	}

	private void activateFocusFrozenHeaderEventHandler(boolean enable) {
		if (enable) {
			activateFocusFrozenHeaderEventHandler(false);
			focusFrozenHeaderEventHandler = $(frozenHeaderId).children("tr")
					.eq(1).children("td").children("div").children("input")
					.focus(new Function() {
						public boolean f(Event event) {
							return SearchingColumnsEvent
									.focus(getThis(), event);
						}
					});
		} else if (!enable && focusFrozenHeaderEventHandler != null) {
			focusFrozenHeaderEventHandler.die();
			focusFrozenHeaderEventHandler = null;
		}
	}

	private void activateBlurFrozenHeaderEventHandler(boolean enable) {
		if (enable) {
			activateBlurFrozenHeaderEventHandler(false);
			blurFrozenHeaderEventHandler = $(frozenHeaderId).children("tr")
					.eq(1).children("td").children("div").children("input")
					.blur(new Function() {
						public boolean f(Event event) {
							return SearchingColumnsEvent.blur(getThis(), event);
						}
					});
		} else if (!enable && blurFrozenHeaderEventHandler != null) {
			blurFrozenHeaderEventHandler.die();
			blurFrozenHeaderEventHandler = null;
		}
	}

	private void activateFrozenHeaderKeydownEventHandler(boolean enable) {
		if (enable) {
			activateFrozenHeaderKeydownEventHandler(false);
			frozenHeaderKeydownEventHandler = $(frozenHeaderId).children("tr")
					.eq(1).children("td").children("div").children("input")
					.keydown(new Function() {
						public boolean f(Event event) {
							return SearchingColumnsEvent.keyDown(getThis(),
									event);
						}
					});
		} else if (!enable && frozenHeaderKeydownEventHandler != null) {
			frozenHeaderKeydownEventHandler.die();
			frozenHeaderKeydownEventHandler = null;
		}
	}

	private void activateFrozenHeaderKeyupEventHandler(boolean enable) {
		if (enable) {
			activateFrozenHeaderKeyupEventHandler(false);
			frozenHeaderKeyupEventHandler = $(frozenHeaderId).children("tr")
					.eq(1).children("td").children("div").children("input")
					.keyup(new Function() {
						public boolean f(Event event) {
							return SearchingColumnsEvent
									.keyUp(getThis(), event);
						}
					});
		} else if (!enable && frozenHeaderKeyupEventHandler != null) {
			frozenHeaderKeyupEventHandler.die();
			frozenHeaderKeyupEventHandler = null;
		}
	}

	private void activateInputMouseOverEventHandler(boolean enable) {
		if (enable) {
			activateInputMouseOverEventHandler(false);
			inputMouseOverEventHandler = $(headerId).children("tr").eq(1)
					.children("td").children("div")
					.delegate("input", "mouseover", new Function() {
						public void f(Element e) {
							SearchingColumnsEvent.mouseOver(getThis(),
									headerId, $(e).id());
						}
					}).delegate("input", "mouseout", new Function() {
						public void f(Element e) {
							SearchingColumnsEvent.mouseOut(getThis(), headerId,
									$(e).id());
						}
					});
		} else if (!enable && inputMouseOverEventHandler != null) {
			inputMouseOverEventHandler.undelegate("input", "mouseover")
					.undelegate("input", "mouseout");
			inputMouseOverEventHandler = null;
		}
	}

	private void activateHeaderMouseOverEventHandler(boolean enable) {
		if (enable) {
			activateHeaderMouseOverEventHandler(false);
			headerMouseOverEventHandler = $(headerId).children("tr").eq(1)
					.delegate("td", "mouseover", new Function() {
						public void f(Element e) {
							SearchingColumnsEvent.mouseOver(getThis(),
									headerId, $(e).id());
						}
					}).delegate("td", "mouseout", new Function() {
						public void f(Element e) {
							SearchingColumnsEvent.mouseOut(getThis(), headerId,
									$(e).id());
						}
					});
		} else if (!enable && headerMouseOverEventHandler != null) {
			headerMouseOverEventHandler.undelegate("td", "mouseover")
					.undelegate("td", "mouseout");
			headerMouseOverEventHandler = null;
		}
	}

	private void activateFocusHeaderEventHandler(boolean enable) {
		if (enable) {
			activateFocusHeaderEventHandler(false);
			focusHeaderEventHandler = $(headerId).children("tr").eq(1)
					.children("td").children("div").children("input")
					.focus(new Function() {
						public boolean f(Event event) {
							return SearchingColumnsEvent
									.focus(getThis(), event);
						}
					});
		} else if (!enable && focusHeaderEventHandler != null) {
			focusHeaderEventHandler.die();
			focusHeaderEventHandler = null;
		}
	}

	private void activateBlurHeaderEventHandler(boolean enable) {
		if (enable) {
			activateBlurHeaderEventHandler(false);
			blurHeaderEventHandler = $(headerId).children("tr").eq(1)
					.children("td").children("div").children("input")
					.blur(new Function() {
						public boolean f(Event event) {
							return SearchingColumnsEvent.blur(getThis(), event);
						}
					});
		} else if (!enable && blurHeaderEventHandler != null) {
			blurHeaderEventHandler.die();
			blurHeaderEventHandler = null;
		}
	}

	private void activateHeaderKeyDownEventHandler(boolean enable) {
		if (enable) {
			activateHeaderKeyDownEventHandler(false);
			headerKeyDownEventHandler = $(headerId).children("tr").eq(1)
					.children("td").children("div").children("input")
					.keydown(new Function() {
						public boolean f(Event event) {
							return SearchingColumnsEvent.keyDown(getThis(),
									event);
						}
					});
		} else if (!enable && headerKeyDownEventHandler != null) {
			headerKeyDownEventHandler.die();
			headerKeyDownEventHandler = null;
		}
	}

	private void activateHeaderKeyUpEventHandler(boolean enable) {
		if (enable) {
			activateHeaderKeyUpEventHandler(false);
			headerKeyUpEventHandler = $(headerId).children("tr").eq(1)
					.children("td").children("div").children("input")
					.keyup(new Function() {
						public boolean f(Event event) {
							return SearchingColumnsEvent
									.keyUp(getThis(), event);
						}
					});
		} else if (!enable && headerKeyUpEventHandler != null) {
			headerKeyUpEventHandler.die();
			headerKeyUpEventHandler = null;
		}
	}

	private void setSearchingColumn(String[] searching) {
		if (searching != null && searching.length > 0) {
			for (int search = 0; search < searching.length; search += 2) {
				columnsSearching.put(Integer.parseInt(searching[search]),
						searching[search + 1]);
			}
		}
	}

	private void fillSearchingColumns() {
		String idFocus = $("*:focus").attr("id");
		for (int column : columnsSearching.keySet()) {
			String header = (getDTable().isFrozen(column) ? frozenHeaderId
					: headerId) + " tr";
			String id = "td [id='" + getDTable().getId() + ":1:" + column
					+ "']";
			String idInput = getDTable().getId() + ":1:" + column + ":2";
			if (!idInput.equals(idFocus)) {
				$(header).eq(1).children(id).children("div").children("input")
						.val(setFieldValue(columnsSearching.get(column)));
			}
		}
		unmarkSearchingField();
	}

	private void removeSearchingColumns() {
		if (!isEmpty()) {
			getDTable().sendMessage("searchingColumns", "none", true);
		}
	}

	private void unmarkSearchingField() {
		if (markedColumn >= 0) {
			focusValue = isSearched(markedColumn) ? columnsSearching
					.get(markedColumn) : new String();
			if (!getDTable().config().isEnabledSearchColumnsImmediate()) {
				String header = (getDTable().isFrozen(markedColumn) ? frozenHeaderId
						: headerId)
						+ " tr";
				String id = "td [id='" + getDTable().getId() + ":1:"
						+ markedColumn + "']";
				$(header).eq(1).children(id).children("div").children("input")
						.blur();
			}
			resetMarkedValue();
		}
	}

	private String getOperator(int column) {
		if (getSearchingColumnValue(column).startsWith("LIKE ")) {
			return "LIKE";
		} else if (getSearchingColumnValue(column).startsWith("NOT LIKE ")) {
			return "NOT LIKE";
		}
		return (getSearchingColumnValue(column).split(" ")[0].trim());
	}

	public void emptySearchingColumn(int column) {
		markedValue = new String();
		initValue = new String();
		focusValue = new String();
		eventValue = new String();

		columnsSearching.remove(column);
		String header = (getDTable().isFrozen(column) ? frozenHeaderId
				: headerId) + " tr";
		String id = "td [id='" + getDTable().getId() + ":1:" + column + "']";
		$(header).eq(1).children(id).children("div").children("input")
				.removeClass("error").val("").blur();
	}

	public boolean filterOperatorExists(int column, String operator) {
		if (getOperator(column).equals(operator)) {
			return true;
		}
		return false;
	}

	public String setFieldValue(String value) {
		if (value.startsWith("= ")) {
			return value.substring("= ".length());
		} else if (value.startsWith("LIKE ")) {
			value = value.substring("LIKE ".length());
			value = value.startsWith("\"") ? value.substring(1) : value;
			value = value.startsWith("'") ? value.substring(1) : value;
			value = value.endsWith("\"") ? value.substring(0,
					value.length() - 1) : value;
			value = value.endsWith("'") ? value
					.substring(0, value.length() - 1) : value;
			return value;
		}
		return value;
	}

	public String getFrozenHeaderId() {
		return frozenHeaderId;
	}

	public String getHeaderId() {
		return headerId;
	}

	public Boolean isSearchingMessageSend() {
		return isSearchingMessageSend;
	}

	public void sendSearchingMessage(Boolean isSearchingMessageSend) {
		this.isSearchingMessageSend = isSearchingMessageSend;
	}

	public String getInitValue() {
		return initValue;
	}

	public void setInitValue(int column) {
		initValue = getSearchingColumnValue(column);
	}

	public String getFocusValue() {
		return focusValue;
	}

	public void setFocusValue(String focusValue) {
		this.focusValue = focusValue;
	}

	public String getEventValue() {
		return eventValue;
	}

	public void setEventValue(String eventValue) {
		this.eventValue = eventValue;
	}

	public boolean isEmpty() {
		return columnsSearching.isEmpty();
	}

	public boolean isSearched(int column) {
		return columnsSearching.containsKey(column);
	}

	public List<Integer> getSearchingColumns() {
		return new ArrayList<Integer>(columnsSearching.keySet());
	}

	public String getSearchingColumnValue(int column) {
		if (isSearched(column)) {
			return columnsSearching.get(column);
		}
		return "";
	}

	public String getMarkedValue() {
		return markedValue;
	}

	public void resetMarkedValue() {
		markedColumn = -1;
		markedValue = new String();
	}

	public void markSearchingField(int column) {
		String header = (getDTable().isFrozen(column) ? frozenHeaderId
				: headerId) + " tr";
		String id = "td [id='" + getDTable().getId() + ":1:" + column + "']";
		eventValue = $(header).eq(1).children(id).children("div")
				.children("input").focus().addClass("error").val();
		focusValue = isSearched(column) ? columnsSearching.get(column)
				: new String();
		markedColumn = column;
		markedValue = eventValue;
	}

	public Boolean isSearchingMarked() {
		return (markedColumn >= 0);
	}

	@Override
	public void setIds() {
		if (!headerId.endsWith("" + getDTable().getId())) {
			frozenHeaderId += getDTable().getId();
			headerId += getDTable().getId();
		}
	}

	public void readUIDL(UIDL uidl) {
		columnsSearching.clear();
		if (uidl.hasAttribute("searchingColumns")) {
			setSearchingColumn(uidl.getStringArrayAttribute("searchingColumns"));
			show = false;
		}

		if (uidl.hasAttribute("removeFiltering")) {
			emptySearchingColumn(uidl.getIntAttribute("removeFiltering"));
		}

		if (uidl.hasAttribute("columnsHeader")
				&& uidl.hasAttribute("headerColumnsAlignment")) {
			show = isVisible;
		}
	}

	@Override
	public void load() {
		setIds();
		isVisible = getDTable().config().isEnabledSearchColumnsVisible();
	}

	@Override
	public void unload() {
		empty();
		disableEventsHandler();
	}

	public void show() {
		if (getDTable().config().isEnabledSearchColumns() && show) {
			prepare();
			appendSearchRows();
			enableEventsHandler();
			show = false;
		}
		fillSearchingColumns();
	}

	public void toggle() {
		if (isVisible) {
			removeSearchingColumns();
			isVisible = false;
			show = false;
			unload();
		} else {
			isVisible = true;
			show = true;
			show();
		}
	}

	@Override
	public void enableEventsHandler() {
		if (getDTable().config().isEnabledSearchColumns()
				&& (getDTable().config().isEnabledFilteredFrozenTable() || getDTable()
						.config().isEnabledFilteredTable())) {
			activateFrozenInputMouseOverEventHandler(true
					&& getDTable().config().isEnabledFrozenTable()
					&& getDTable().config()
							.isEnabledExpandedFrozenHeaderCellForActiveColumn());
			activateFrozenHeaderMouseOverEventHandler(true
					&& getDTable().config().isEnabledFrozenTable()
					&& getDTable().config()
							.isEnabledExpandedFrozenHeaderCellForActiveColumn());
			activateFocusFrozenHeaderEventHandler(true && getDTable().config()
					.isEnabledFrozenTable());
			activateBlurFrozenHeaderEventHandler(true && getDTable().config()
					.isEnabledFrozenTable());
			activateFrozenHeaderKeydownEventHandler(true && getDTable()
					.config().isEnabledFrozenTable());
			activateFrozenHeaderKeyupEventHandler(true && getDTable().config()
					.isEnabledFrozenTable());
			activateInputMouseOverEventHandler(true && getDTable().config()
					.isEnabledExpandedHeaderCellForActiveColumn());
			activateHeaderMouseOverEventHandler(true && getDTable().config()
					.isEnabledExpandedHeaderCellForActiveColumn());
			activateFocusHeaderEventHandler(true);
			activateBlurHeaderEventHandler(true);
			activateHeaderKeyDownEventHandler(true);
			activateHeaderKeyUpEventHandler(true);
		}
	}

	@Override
	public void disableEventsHandler() {
		activateFrozenInputMouseOverEventHandler(false);
		activateFrozenHeaderMouseOverEventHandler(false);
		activateFocusFrozenHeaderEventHandler(false);
		activateBlurFrozenHeaderEventHandler(false);
		activateFrozenHeaderKeydownEventHandler(false);
		activateFrozenHeaderKeyupEventHandler(false);
		activateInputMouseOverEventHandler(false);
		activateHeaderMouseOverEventHandler(false);
		activateFocusHeaderEventHandler(false);
		activateBlurHeaderEventHandler(false);
		activateHeaderKeyDownEventHandler(false);
		activateHeaderKeyUpEventHandler(false);
	}

}
