/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.client.components.events;

import static com.google.gwt.query.client.GQuery.$;

import org.dussan.vaadin.dtable.client.components.SearchingColumns;

import com.google.gwt.event.dom.client.KeyCodes;
import com.google.gwt.user.client.Event;

public class SearchingColumnsEvent {

	private static boolean canSendSearch(SearchingColumns searchColumns,
			int column, String searchString) {
		if ("".equals(searchString.trim()) && !searchString.isEmpty()) {
			return false;
		} else if (searchColumns.getDTable().config().isColumnFormatted(column)
				&& !"".equals(searchString.replaceAll("\\s+", " ").trim())) {
			String search = searchString.replaceAll("\\s+", " ").trim();
			while (search.startsWith("=") || search.startsWith("<")
					|| search.startsWith(">") || search.startsWith("-")
					|| search.startsWith("+") || search.startsWith(",")
					|| search.startsWith(".")) {
				search = search.substring(1);
			}
			return (search.trim().length() > 0);
		} else if (!searchColumns.getDTable().config()
				.isColumnFormatted(column)
				&& !"".equals(searchString.replaceAll("\\s+", " ").trim())) {
			String search = searchString.replaceAll("\\s+", " ").trim();
			while (search.startsWith("%") || search.startsWith("\"")
					|| search.startsWith("'")) {
				search = search.substring(1);
			}
			return (search.trim().length() > 0);
		}
		return true;
	}

	private static void checkSearchField(SearchingColumns searchColumns,
			int column, Event event) {
		if (!canSendSearch(searchColumns, column, $(event).val())) {
			$(event).addClass("error");
		} else if (searchColumns.isSearchingMarked()
				&& !searchColumns.getMarkedValue().isEmpty()
				&& searchColumns.getMarkedValue().equals($(event).val())) {
			$(event).addClass("error");
		} else {
			$(event).removeClass("error");
		}
	}

	private static String prepareSearchMessage(SearchingColumns searchColumns,
			int column, String searchString) {
		if (searchString != null && !searchString.trim().isEmpty()) {
			searchString = searchString.replaceAll("\\s+", " ").trim();
			if (searchColumns.getDTable().config().isColumnFormatted(column)) {
				if (searchString.startsWith("<=")
						|| searchString.startsWith(">=")
						|| searchString.startsWith("<>")) {
					searchString.replace("<=", "<= ").replace(">=", ">= ")
							.replace("<>", "<> ");
				} else if (searchString.startsWith("=")
						|| searchString.startsWith("<")
						|| searchString.startsWith(">")) {
					searchString.replace("<", "< ").replace(">", "> ")
							.replace("=", "= ");
				} else {
					searchString = "= " + searchString;
				}
			}
			searchString = searchString.replaceAll("\\s+", " ").trim();
		}
		return (column + "_::_" + searchString);
	}

	public static void mouseOver(SearchingColumns searchColumns,
			String headerId, String id) {
		// expanded header cell
		String[] ids = id.split(":");
		TableEvent.cellMouseOver(searchColumns.getDTable(), $(headerId + " tr")
				.children("td [id='" + ids[0] + "h:0:" + ids[2] + "']"), null);
	}

	public static void mouseOut(SearchingColumns searchColumns,
			String headerId, String id) {
		// expanded header cell
		String[] ids = id.split(":");
		TableEvent.headerCellMouseOut(headerId + " tr", ids[0], ids[2], null);
	}

	public static boolean keyDown(SearchingColumns searchColumns, Event event) {
		if (!searchColumns.getDTable().sendingMessage()) {
			if (event.getKeyCode() == KeyCodes.KEY_TAB) {
				$(event).blur();
				return false;
			}
		}
		return true;
	}

	public static boolean keyUp(SearchingColumns searchColumns, Event event) {
		if (!searchColumns.getDTable().sendingMessage()) {
			String[] id = $(event).id().split(":");
			if (id != null && id.length == 4 && id[3].equals("2")) {
				int column = Integer.parseInt(id[2]);
				String value = $(event).val();
				checkSearchField(searchColumns, column, event);

				if (event.getKeyCode() == KeyCodes.KEY_ESCAPE) {
					searchColumns.setFocusValue(searchColumns.getInitValue());
					$(event).blur();
				} else if (event.getKeyCode() == KeyCodes.KEY_ENTER) {
					if (!searchColumns.getDTable().config()
							.isEnabledSearchColumnsImmediate()
							&& !searchColumns.isSearchingMessageSend()
							&& canSendSearch(searchColumns, column, value)
							&& !searchColumns.getFocusValue().equals(value)
							&& (!searchColumns.getEventValue().equals(value) || ""
									.equals(searchColumns.getEventValue()))) {
						searchColumns.sendSearchingMessage(true);
						searchColumns.setEventValue(value);
						searchColumns.getDTable().sendMessage(
								"searchingColumns",
								prepareSearchMessage(searchColumns, column,
										value), true);
					}

					if (!searchColumns.isSearchingMarked()) {
						if (canSendSearch(searchColumns, column, value)) {
							if (searchColumns.getDTable().config()
									.isEnabledSearchColumnsImmediate()) {
								value = searchColumns.getSearchingColumnValue(column);
								searchColumns.setEventValue(value);
							}
							searchColumns.setFocusValue(value);
						}
						$(event).blur();
					}
				} else if (searchColumns.getDTable().config()
						.isEnabledSearchColumnsImmediate()
						&& event.getKeyCode() != KeyCodes.KEY_CTRL
						&& event.getKeyCode() != KeyCodes.KEY_ALT
						&& event.getKeyCode() != KeyCodes.KEY_LEFT
						&& event.getKeyCode() != KeyCodes.KEY_RIGHT
						&& event.getKeyCode() != KeyCodes.KEY_UP
						&& event.getKeyCode() != KeyCodes.KEY_DOWN
						&& canSendSearch(searchColumns, column, value)) {
					if (!searchColumns.getEventValue().equals(value)) {
						searchColumns.sendSearchingMessage(true);
						searchColumns.setEventValue(value);
						searchColumns.getDTable().sendMessage(
								"searchingColumns",
								prepareSearchMessage(searchColumns, column,
										value), true);
					}
				}
				return false;
			}
		}
		return true;
	}

	public static boolean focus(SearchingColumns searchColumns, Event event) {
		String[] id = $(event).id().split(":");
		if (id != null && id.length == 4 && id[3].equals("2")) {
			int column = Integer.parseInt(id[2]);
			if (searchColumns.getDTable().isFrozen(column)
					&& !searchColumns.getDTable().config()
							.isEnabledFilteredFrozenTable()) {
				$(event).blur();
				return true;
			}
			searchColumns.setInitValue(column);
			searchColumns.setFocusValue($(event).val());
			searchColumns.setEventValue($(event).val());
			searchColumns.sendSearchingMessage(false);
			searchColumns.resetMarkedValue();

			// expanded header cell
			String tr = null;
			if (searchColumns.getDTable().config()
					.isEnabledExpandedFrozenHeaderCellForActiveColumn()
					&& searchColumns.getDTable().isFrozen(column)) {
				tr = searchColumns.getFrozenHeaderId() + " tr";
			} else if (searchColumns.getDTable().config()
					.isEnabledExpandedHeaderCellForActiveColumn()
					&& !searchColumns.getDTable().isFrozen(column)) {
				tr = searchColumns.getHeaderId() + " tr";
			}
			if (tr != null) {
				TableEvent.headerCellMouseOut(tr, id[0], id[2], null);
				TableEvent.cellMouseOver(searchColumns.getDTable(), $(tr)
						.children("td [id='" + id[0] + "h:0:" + id[2] + "']"),
						"focus");
			}
		}
		return true;
	}

	public static boolean blur(SearchingColumns searchColumns, Event event) {
		if ($(event).id() != null && $(event).id().split(":").length == 4) {
			String[] id = $(event).id().split(":");
			if (id[3].equals("2")) {
				String value = searchColumns.getFocusValue();
				if (searchColumns.isSearchingMessageSend()
						&& !value.equals(searchColumns.getEventValue())) {
					searchColumns.setEventValue(value);
					searchColumns.getDTable().sendMessage(
							"searchingColumns",
							prepareSearchMessage(searchColumns,
									Integer.parseInt(id[2]), value), true);
				}
				$(event).val(searchColumns.setFieldValue(value));
			}
			$(event).removeClass("error");

			// expanded header cell
			String tr = null;
			int column = Integer.parseInt(id[2]);
			if (searchColumns.getDTable().config()
					.isEnabledExpandedFrozenHeaderCellForActiveColumn()
					&& searchColumns.getDTable().isFrozen(column)) {
				tr = searchColumns.getFrozenHeaderId() + " tr";
			} else if (searchColumns.getDTable().config()
					.isEnabledExpandedHeaderCellForActiveColumn()
					&& !searchColumns.getDTable().isFrozen(column)) {
				tr = searchColumns.getHeaderId() + " tr";
			}
			if (tr != null) {
				TableEvent.headerCellMouseOut(tr, id[0], id[2], "focus");
			}
		}
		return true;
	}

}
