/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.processors;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.dussan.vaadin.dtable.properties.filtering.Filtering;

public class FilteringProcessor {

	public static Filtering process(String[] newFilter, Filtering filtering,
			Object originalValue, String header) {
		if (newFilter != null && newFilter.length > 0) {
			Integer column = Integer.parseInt(newFilter[0]);
			List<String> operators = new ArrayList<String>();
			List<String> values = new ArrayList<String>();
			List<String> originalValues = new ArrayList<String>();

			if (filtering != null) {
				operators.addAll(Arrays.asList(filtering.getOperators()));
				values.addAll(Arrays.asList(filtering.getValues()));
				originalValues.addAll(Arrays.asList(filtering
						.getOriginalValues()));
			}

			String newOperator = newFilter[2];
			String newValue = newFilter[4];
			String newOriginalValue = originalValue == null ? new String()
					: originalValue.toString();
			if (newOperator.endsWith("EMPTY")) {
				newValue = "";
				newOriginalValue = "";
			} else if (newOperator.endsWith("LIKE")) {
				newValue = newFilter[3].substring(newOperator.length())
						.replace("'", "").trim();
				newOriginalValue = newValue;
			}
			operators.add(newOperator);
			values.add(newValue);
			originalValues.add(newOriginalValue);

			return new Filtering(column, header,
					operators.toArray(new String[] {}),
					values.toArray(new String[] {}),
					originalValues.toArray(new String[] {}));
		}
		return null;
	}

}
